@@ -44,7 +44,6 @@ next_bday  <- next_businessday <-
         x[tmpi] <- x[tmpi] + 2L
         tmpi <- tmp$wday == 0L
         x[tmpi] <- x[tmpi] + 1L
-
     } else {
         for (i in 1:shift) {
             x <- x + 1
@@ -270,6 +269,8 @@ ssm <- function(time, tz = "") {
 
 convert_date <- function(x, type, fraction = FALSE, tz = "") {
     type <- tolower(type)
+    if (type == "spss")
+        type = "pspp"
     if (type == "excel" && !fraction){
         as.Date(x, origin = "1899-12-30")
     } else if (type == "excel") {
@@ -280,6 +281,11 @@ convert_date <- function(x, type, fraction = FALSE, tz = "") {
     } else if (type == "matlab" && fraction) {
         tmp <- as.POSIXct((x - 719529) * 86400, origin = "1970-01-01")
         as.POSIXct(strptime(format(tmp), format = "%Y-%m-%d %H:%M:%S", tz = tz))
+    } else if (type == "pspp" && !fraction) {
+        as.Date(x/86400, origin = "1582-10-14")
+    } else if (type == "pspp" && fraction) {
+        tmp <- as.POSIXct(x, origin = "1582-10-14", tz = "UTC")
+        as.POSIXct(strptime(format(tmp), format = "%Y-%m-%d %H:%M:%S"))
     } else
         stop("unknown type")
 }
@@ -436,32 +442,38 @@ nth_day <- function(timestamps,
         timestamps[ii]
 }
 
+
+## matching years: try "Y" first and then "y"
+
 .dt_patterns <- c(
-    "[1-2][0-9][0-9][0-9]-[0-9][0-9]?-[0-9][0-9]? +[0-9]+:[0-9]+:[0-9]+", "%Y-%m-%d %H:%M:%S",
-    "[0-9][0-9]+-[0-9]+-[0-9]+ +[0-9]+:[0-9]+",                           "%Y-%m-%d %H:%M",
-    "[1-2][0-9][0-9][0-9][0-9][0-9][0-9][0-9] +[0-9]+:[0-9]+:[0-9]+",     "%Y%m%d %H:%M:%S",
-    "[1-2][0-9][0-9][0-9][0-9][0-9][0-9][0-9] +[0-9]+:[0-9]+",            "%Y%m%d %H:%M",
-    "[0-9]+/[0-9]+/[0-9][0-9] +[0-9]+:[0-9]+:[0-9]+",                     "%m/%d/%y %H:%M:%S",
-    "[0-9]+/[0-9]+/[1-2][0-9][0-9][0-9] +[0-9]+:[0-9]+:[0-9]+",           "%m/%d/%Y %H:%M:%S",
-    "[0-9]+/[0-9]+/[0-9][0-9] +[0-9]+:[0-9]+",                            "%m/%d/%y %H:%M",
-    "[0-9]+/[0-9]+/[1-2][0-9][0-9][0-9] +[0-9]+:[0-9]+",                  "%m/%d/%Y %H:%M",
-    "[0-9][0-9]*[.][0-9]+[.][1-2][0-9][0-9][0-9] +[0-9]+:[0-9]+:[0-9]+",  "%d.%m.%Y %H:%M:%S",
-    "[0-9][0-9]*[.][0-9]+[.][1-2][0-9][0-9][0-9] +[0-9]+:[0-9]+",         "%d.%m.%Y %H:%M"
+    "[1-2][0-9][0-9][0-9]-[0-9][0-9]?-[0-9][0-9]? +[0-9]+:[0-9]+:[0-9]+",         "%Y-%m-%d %H:%M:%S",
+    "[0-9][0-9]+-[0-9]+-[0-9]+ +[0-9]+:[0-9]+",                                   "%Y-%m-%d %H:%M",
+    "[1-2][0-9][0-9][0-9][0-9][0-9][0-9][0-9] +[0-9]+:[0-9]+:[0-9]+",             "%Y%m%d %H:%M:%S",
+    "[1-2][0-9][0-9][0-9][0-9][0-9][0-9][0-9] +[0-9]+:[0-9]+",                    "%Y%m%d %H:%M",
+    "[0-9]+/[0-9]+/[1-2][0-9][0-9][0-9] +[0-9]+:[0-9]+:[0-9]+",                   "%m/%d/%Y %H:%M:%S",
+    "[0-9]+/[0-9]+/[0-9][0-9] +[0-9]+:[0-9]+:[0-9]+",                             "%m/%d/%y %H:%M:%S",
+    "[0-9]+/[0-9]+/[1-2][0-9][0-9][0-9] +[0-9]+:[0-9]+",                          "%m/%d/%Y %H:%M",
+    "[0-9]+/[0-9]+/[0-9][0-9] +[0-9]+:[0-9]+",                                    "%m/%d/%y %H:%M",
+    "[0-9][0-9]{1,2}[.][0-9]{1,2}[.][1-2][0-9][0-9][0-9] +[0-9]+:[0-9]+:[0-9]+",  "%d.%m.%Y %H:%M:%S",
+    "[0-9][0-9]{1,2}[.][0-9]{1,2}[.][0-9][0-9] +[0-9]+:[0-9]+:[0-9]+",            "%d.%m.%y %H:%M:%S",
+    "[0-9][0-9]{1,2}[.][0-9]{1,2}[.][1-2][0-9][0-9][0-9] +[0-9]+:[0-9]+",         "%d.%m.%Y %H:%M",
+    "[0-9][0-9]{1,2}[.][0-9]{1,2}[.][0-9][0-9] +[0-9]+:[0-9]+",                   "%d.%m.%y %H:%M"
 )
 
 .d_patterns <- c(
-    "[1-2][0-9][0-9][0-9]-[0-9]+-[0-9]+",           "%Y-%m-%d",
-    "[0-9][0-9]+-[0-9]+-[0-9]+",                    "%Y-%m-%d",
-    "[1-2][0-9][0-9][0-9][0-9][0-9][0-9][0-9]",     "%Y%m%d",
-    "[0-9]+/[0-9]+/[0-9][0-9]",                     "%m/%d/%y",
-    "[0-9]+/[0-9]+/[1-2][0-9][0-9][0-9]",           "%m/%d/%Y",
-    "[0-9]+/[0-9]+/[0-9][0-9]",                     "%m/%d/%y",
-    "[0-9]+/[0-9]+/[1-2][0-9][0-9][0-9]",           "%m/%d/%Y",
-    "[0-9][0-9]*[.][0-9]+[.][1-2][0-9][0-9][0-9]",  "%d.%m.%Y",
-    "[0-9][0-9]*[.][0-9]+[.][1-2][0-9][0-9][0-9]",  "%d.%m.%Y"
+    "[1-2][0-9][0-9][0-9]-[0-9][0-9]?-[0-9][0-9]?",         "%Y-%m-%d",
+    "[0-9][0-9]?-[0-9][0-9]?-[0-9][0-9]?",                  "%Y-%m-%d",
+    "[1-2][0-9][0-9][0-9][0-9][0-9][0-9][0-9]",             "%Y%m%d",
+    "[01]?[0-9]/[0-9]+/[1-2][0-9][0-9][0-9]",               "%m/%d/%Y",
+    "[01]?[0-9]/[0-9]+/[0-9][0-9]",                         "%m/%d/%y",
+    "[01]?[0-9]/[0-9]+/[1-2][0-9][0-9][0-9]",               "%m/%d/%Y",
+    "[01]?[0-9]/[0-9]+/[0-9][0-9]",                         "%m/%d/%y",
+    "[0-9][0-9]{1,2}[.][0-9]{1,2}[.][1-2][0-9][0-9][0-9]",  "%d.%m.%Y",
+    "[0-9][0-9]{1,2}[.][0-9]{1,2}[.][1-2][0-9]",            "%d.%m.%y"
 )
 
-guess_datetime <- function(s, date.only = FALSE, within = FALSE, tz = "") {
+guess_datetime <- function(s, date.only = FALSE, within = FALSE, tz = "",
+                           try.patterns = NULL) {
 
     x <- as.character(s)
     patterns <- if (date.only)
@@ -469,6 +481,9 @@ guess_datetime <- function(s, date.only = FALSE, within = FALSE, tz = "") {
                 else
                     .dt_patterns
 
+    if (!is.null(try.patterns))
+        patterns <- c(try.patterns, patterns)
+
     ans <- if (date.only)
                .Date(rep(NA_real_, length(x)))
            else
@@ -479,13 +494,15 @@ guess_datetime <- function(s, date.only = FALSE, within = FALSE, tz = "") {
     ii <- seq(1, length(patterns), by = 2L)
     for (t in ii) {
         i <- grepl(patterns[t], x) & !done
+        if (!any(i))
+            next
         tmp <- x[i]
         if (within)
-            tmp <- gsub(paste0(".*(", patterns[t], ").*"), "\\1", tmp)
+            tmp <- gsub(paste0(".*?(", patterns[t], ").*"), "\\1", tmp)
         ans[i] <- if (date.only)
                       as.Date   (strptime(tmp, patterns[t + 1L]))
                   else
-                      as.POSIXct(strptime(tmp, patterns[t + 1L]), tz = "")
+                      as.POSIXct(strptime(tmp, patterns[t + 1L]), tz = tz)
         done[i] <- TRUE
 
         if (all(done))
@@ -509,6 +526,27 @@ month <- function(x, as.character = FALSE) {
         as.POSIXlt(x)$mon + 1
 }
 
+hour <- function(x, as.character = FALSE) {
+    if (as.character)
+        as.character(as.POSIXlt(Sys.time())$hour)
+    else
+        as.POSIXlt(Sys.time())$hour
+}
+
+minute <- function(x, as.character = FALSE) {
+    if (as.character)
+        as.character(as.POSIXlt(Sys.time())$min)
+    else
+        as.POSIXlt(Sys.time())$min
+}
+
+second <- function(x, as.character = FALSE) {
+    if (as.character)
+        as.character(as.POSIXlt(Sys.time())$sec)
+    else
+        as.POSIXlt(Sys.time())$sec
+}
+
 .weekday <- function(dates)
     unclass(dates + 4) %% 7
 
